#include "cpu_speed.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define ClearBit(PORT, BIT)   (PORT &= ~(1 << BIT))
#define SetBit(PORT, BIT)     (PORT |= (1 << BIT))
//#define RF02FREQ(freq)	      ((freq-868.0)/0.005)
#define FREQ_C1               2   // 1=433MHz, 2=868MHz, 3=915MHz
#define FREQ_C2               43  // 43 = 433MHz/ 868MHz, 30 = 915MHz
#define RF02FREQ(freq)	      (4000*((freq/10*FREQ_C1)-FREQ_C2))

// RFM02 Inputs (= AVR Outputs)
//#define RF02_FSK_PORT   PORTD
//#define RF02_FSK_DDR    DDRD
//#define RF02_FSK        PD7
#define RF02_SDI_PORT   PORTB
#define RF02_SDI_DDR    DDRB
#define RF02_SDI        PB3
#define RF02_SCK_PORT   PORTB
#define RF02_SCK_DDR    DDRB
#define RF02_SCK        PB5
#define RF02_NSEL_PORT  PORTB
#define RF02_NSEL_DDR   DDRB
#define RF02_NSEL       PB2

// RFM02 Outputs (= AVR Inputs)
//#define RF02_CLK_PORT   ???     // Not connected on Pollin FUNK-AVR-Evaluationsboard
//#define RF02_CLK_DDR    ???     // Not connected on Pollin FUNK-AVR-Evaluationsboard
//#define RF02_CLK_PIN    ???     // Not connected on Pollin FUNK-AVR-Evaluationsboard
//#define RF02_CLK        ???     // Not connected on Pollin FUNK-AVR-Evaluationsboard
#define RF02_IRQ_PORT   PORTD
#define RF02_IRQ_DDR    DDRD
#define RF02_IRQ_PIN    PIND
#define RF02_IRQ        PD2

// Evaluationsboard Outputs
#define EVA_LED1_PORT   PORTD
#define EVA_LED1_DDR    DDRD
#define EVA_LED1        PD6
//#define EVA_LED2_PORT   PORTD
//#define EVA_LED2_DDR    DDRD
//#define EVA_LED2        PD5

// Evaluationsboard Inputs
//#define EVA_BTN1_PORT   PORTB
//#define EVA_BTN1_DDR    DDRB
//#define EVA_BTN1_PIN    PINB
//#define EVA_BTN1        PB1


void rf02_trans(unsigned short wert);
void rf02_init(void);							// init functions
void rf02_setmodfreq(unsigned char bandwidth);	// set modulation deviation
void rf02_setfreq(unsigned short freq);			// set tx frequency
void rf02_setpower(unsigned char power);			// set power (0-7)
void rf02_setbaud(unsigned short baud);			// set baudrate
void rf02_txdata(unsigned char *data, unsigned char number);	// tx data
void rf02_shiftout(unsigned char wert);

// FSK: Pullup oder Pulldown

void rf02_trans(unsigned short wert)
{	
  unsigned char i;

	//cbi(RF_PORT, CS);
  ClearBit(RF02_NSEL_PORT, RF02_NSEL);
	for (i=0; i<16; i++)
	{
  	//if (wert&32768)
    if (wert&0x8000)
    {
			//sbi(RF_PORT, SDI);
      SetBit(RF02_SDI_PORT, RF02_SDI);
    }
		else
    {
			//cbi(RF_PORT, SDI);
      ClearBit(RF02_SDI_PORT, RF02_SDI);
    }
		
    //sbi(RF_PORT, SCK);
    SetBit(RF02_SCK_PORT, RF02_SCK);

		wert<<=1;
		_delay_us(0.3);
		
    //cbi(RF_PORT, SCK);
    ClearBit(RF02_SCK_PORT, RF02_SCK);
	}
	
  //sbi(RF_PORT, CS);
  SetBit(RF02_NSEL_PORT, RF02_NSEL);
}

void rf02_init(void)
{
  // Set used PORTS to output
  RF02_NSEL_DDR |= (1<<RF02_NSEL);
  RF02_SDI_DDR  |= (1<<RF02_SDI);
  RF02_SCK_DDR  |= (1<<RF02_SCK);
  
  // Set used PORTS to input
  RF02_IRQ_DDR  &= ~(1<<RF02_IRQ);

	//Set output ports
  RF02_NSEL_PORT |= (1<<RF02_NSEL);

	//Enable pullups at input pins
  RF02_IRQ_PORT  |= (1<<RF02_IRQ);

	for (unsigned char i=0; i<15; i++)
		_delay_ms(10);			// wait until POR done
	rf02_trans(0xC0E0);			// power settings
	//rf02_trans(0x8F80); // Configuration Setting command for 433MHz
  rf02_trans(0x9780); // Configuration Setting command for 868MHz
	rf02_trans(0xC2A0);			// enable tx sync bit, disable low bat detector
}

void rf02_setmodfreq(unsigned char bandwidth)
{
	rf02_trans(0x8F80|(bandwidth&7));
}

void rf02_setfreq(unsigned short freq)
{	
  if (freq<96)				// 430,2400MHz | 860,4800MHz | 900,7200MHz
		freq=96;
	else if (freq>3903)			// 439,7575MHz | 879,5100MHz | 929,2700MHz
		freq=3903;
	rf02_trans(0xA000|freq);
}

void rf02_setpower(unsigned char power)
{
	rf02_trans(0xB000|((power&7)<<8));
}

void rf02_setbaud(unsigned short baud)
{
	if (baud<1345)
		baud=1345;
	if (baud<19000)
		rf02_trans(0xD240);		// 25% PLL current
	else if (baud<37000)
		rf02_trans(0xD2C0);		// 33% PLL current
	else
		rf02_trans(0xD200);		// 50% PLL current
	rf02_trans(0xC800|((344828UL/baud)-1));	// Baudrate= 344827,59/(R+1)
}

void rf02_txdata(unsigned char *data, unsigned char number)
{
	unsigned char i,wert;
	wert=0xC6;
	//cbi(RF_PORT, CS);
  ClearBit(RF02_NSEL_PORT, RF02_NSEL);

	for (i=0; i<8; i++)
	{	
    //if (wert&128)
    if (wert&0x80)
    {
			//sbi(RF_PORT, SDI);
      SetBit(RF02_SDI_PORT, RF02_SDI);
    }
		else
    {
			//cbi(RF_PORT, SDI);
      ClearBit(RF02_SDI_PORT, RF02_SDI);
    }

		//sbi(RF_PORT, SCK);
    SetBit(RF02_SCK_PORT, RF02_SCK);

		wert<<=1;
		_delay_us(0.2);
		//cbi(RF_PORT, SCK);
    ClearBit(RF02_SCK_PORT, RF02_SCK);
	}

  rf02_shiftout(0xAA);
	rf02_shiftout(0xAA);
	rf02_shiftout(0xAA);
	rf02_shiftout(0x2D);
	rf02_shiftout(0xD4);
	
  for (i=0; i<number; i++)
  {
		rf02_shiftout(*data++);
  }

	//sbi(RF_PORT, CS);
  SetBit(RF02_NSEL_PORT, RF02_NSEL);

	//while(RF_PIN&(1<<IRQ));
  while(RF02_IRQ_PIN&(1<<RF02_IRQ))
  {
    // wait until transfer done
  }
	
  rf02_trans(0xC464);			// TX off after 10us
}

void rf02_shiftout(unsigned char wert)
{	
  unsigned char j;
	for (j=0; j<8; j++)
	{	
    //while(RF_PIN&(1<<IRQ));
    while(RF02_IRQ_PIN&(1<<RF02_IRQ))
    {
    }
    	
    //while(!(RF_PIN&(1<<IRQ)));
    while(!(RF02_IRQ_PIN&(1<<RF02_IRQ)))
    {
    }

		//if (wert&128)
    if (wert&0x80)
    {
      //sbi(RF_PORT, SDI);
      SetBit(RF02_SDI_PORT, RF02_SDI);
    }
    else
    {
   		//cbi(RF_PORT, SDI);
      ClearBit(RF02_SDI_PORT, RF02_SDI);
    }
    wert<<=1;
  }
}


//unsigned char test[32]="Dies ist ein 433MHz Test !!!\n   ";
unsigned char test[32]="Dies ist ein 868MHz Test !!!\n   ";

int main(void)
{
  SetBit(EVA_LED1_PORT, EVA_LED1);
  SetBit(EVA_LED1_DDR,  EVA_LED1);

  char cKanal = 4; // Die Kanle 4, 7, 10 und 13 drfen ohne Verwendung von LBT genutzt werden (also 100% Senden)

	rf02_init();					// ein paar Register setzen (z.B. CLK auf 10MHz)
	//rf02_setfreq(RF02FREQ(433.92));	// Sende/Empfangsfrequenz auf 433,92MHz einstellen
  rf02_setfreq(RF02FREQ(864.9 + (0.2*cKanal)));	// Sende/Empfangsfrequenz auf 868MHz, Kanal 4 einstellen
	rf02_setpower(4);				// -12dBm Ausgangangsleistung
	rf02_setmodfreq(3);				// 120kHz Frequenzshift
	rf02_setbaud(19200);			// 19200 Baud
	for (;;)
	{
		rf02_txdata(test,32);
		for (unsigned char i=0; i<100; i++)
    {
		  _delay_ms(10);
    }

    EVA_LED1_PORT ^= (1<<EVA_LED1);
	}
}
