#include "cpu_speed.h"
#include <avr/io.h>

//#include <avr/interrupt.h>
//#include <avr/pgmspace.h>
//#include <avr/eeprom.h>
//#include <stdlib.h>
//#include "global.h"
//#include <util/delay.h>

#define DDR_IN      0
#define DDR_OUT     1

#define PORT_SEL    PORTB
#define PIN_SEL     PINB
#define DDR_SEL     DDRB

#define PORT_SDI    PORTB
#define PIN_SDI     PINB
#define DDR_SDI     DDRB

#define PORT_SCK    PORTB
#define PIN_SCK     PINB
#define DDR_SCK     DDRB

#define PORT_SDO    PORTB
#define PIN_SDO     PINB
#define DDR_SDO     DDRB

//#define PB7
//#define PB6
#define RFXX_SCK    PB5
//#define RFXX_SDO    PB4
#define RFXX_SDI    PB3
#define RFXX_SEL    PB2
//#define RFXX_DATA   PD7
//#define PB0

#define SEL_OUTPUT()    DDR_SEL   |=  (1<<RFXX_SEL)
#define HI_SEL()        PORT_SEL  |=  (1<<RFXX_SEL)
#define LOW_SEL()       PORT_SEL  &= ~(1<<RFXX_SEL)

#define SDI_OUTPUT()    DDR_SDI   |=  (1<<RFXX_SDI)
#define HI_SDI()        PORT_SDI  |=  (1<<RFXX_SDI)
#define LOW_SDI()       PORT_SDI  &= ~(1<<RFXX_SDI)

//#define SDO_INPUT()     DDR_SDO   &= ~(1<<RFXX_SDO)
//#define SDO_HI()        PIN_SDO   & (1<<RFXX_SDO)

#define SCK_OUTPUT()    DDR_SCK   |=  (1<<RFXX_SCK)
#define HI_SCK()        PORT_SCK  |=  (1<<RFXX_SCK)
#define LOW_SCK()       PORT_SCK  &= ~(1<<RFXX_SCK)

void RFXX_PORT_INIT(void)
{
  HI_SEL();
  HI_SDI();
  LOW_SCK();
  SEL_OUTPUT();
  SDI_OUTPUT();
  //SDO_INPUT();
  SCK_OUTPUT();
}

unsigned int RFXX_WRT_CMD(unsigned int aCmd)
{
  unsigned char i;
  unsigned int temp;
  LOW_SCK();
  LOW_SEL();
  for(i = 0; i < 16; i++)
  {
    temp <<= 1;
    if (SDO_HI())
    {
      temp |= 0x0001;
    }
    LOW_SCK();
    if(aCmd & 0x8000)
    {
      HI_SDI();
    }
    else
    {
      LOW_SDI();
    }
    HI_SCK();
    aCmd <<= 1;
  }
  LOW_SCK();
  HI_SEL();
  return (temp);
}

void RF02B_SEND(unsigned char aByte)
{
  unsigned char i;

  for (i = 0; i < 8; i++)
  {
    //while (PINB&(1<<RFXX_SDO));   // Polling nIRQ
    //while (!(PINB&(1<<RFXX_SDO)));
    if (aByte && 0x80)
    {
      PORTB |= (1<<RFXX_DATA);
    }
    else
    {
      PORTB &= ~(1<<RFXX_DATA);
    }
    aByte <<= 1;
  }
}

void main (void)
{
  // LED
  DDRB = 0xFF;
  PORTD = (1<<PD5);

  unsigned int i, j, ChkSum;

  RFXX_PORT_INIT();

  RFXX_WRT_CMD(0xCC00);
  RFXX_WRT_CMD(0x8B81); // 433BAND, +/- 60kHz
  RFXX_WRT_CMD(0xA640); // 434MHz;
  RFXX_WRT_CMD(0xC847); // 4,8kbps
  RFXX_WRT_CMD(0xC220); // ENABLE BIT SYNC
  RFXX_WRT_CMD(0xC001); // CLOSE ALL

  PORTB |= (1<<RFXX_DATA);
  DDRB  |= (1<<RFXX_DATA);  // SET DATA OUTPUT

  while (1)
  {
    RFXX_WRT_CMD(0xC039);   // START TX
    ChkSum = 0;
    RFX02B_SEND(0xAA);  // PREAMBLE
    RFX02B_SEND(0xAA);  // PREAMBLE
    RFX02B_SEND(0xAA);  // PREAMBLE
    RFX02B_SEND(0x2D);  // HEAD HI BYTE
    RFX02B_SEND(0xD4);  // HEAD LOW BYTE
    RFX02B_SEND(0x30);  // DATA 0
    ChkSum += 0x30;
    RFX02B_SEND(0x31);  // DATA 1
    ChkSum += 0x31;
    RFX02B_SEND(0x32);  // DATA 2
    ChkSum += 0x32;
    RFX02B_SEND(0x33);  // DATA 3
    ChkSum += 0x33;
    RFX02B_SEND(0x34);  // DATA 4
    ChkSum += 0x34;
    RFX02B_SEND(0x35);  // DATA 5
    ChkSum += 0x35;
    RFX02B_SEND(0x36);  // DATA 6
    ChkSum += 0x36;
    RFX02B_SEND(0x37);  // DATA 7
    ChkSum += 0x37;
    RFX02B_SEND(0x38);  // DATA 8
    ChkSum += 0x38;
    RFX02B_SEND(0x39);  // DATA 9
    ChkSum += 0x39;
    RFX02B_SEND(0x3A);  // DATA 10
    ChkSum += 0x3A;
    RFX02B_SEND(0x3B);  // DATA 11
    ChkSum += 0x3B;
    RFX02B_SEND(0x3C);  // DATA 12
    ChkSum += 0x3C;
    RFX02B_SEND(0x3D);  // DATA 13
    ChkSum += 0x3D;
    RFX02B_SEND(0x3E);  // DATA 14
    ChkSum += 0x3E;
    RFX02B_SEND(0x3F);  // DATA 15
    ChkSum += 0x3F;
    RFX02B_SEND(ChkSum);// DATA 16
    RFX02B_SEND(0xAA);  // DUMMY BYTE

    RFXX_WRT_CMD(0xC001); // CLOSE TX

    PORTD ^= (1<<PD5);
    _delay_ms(1000);

  }
}


